import uvicorn
from apscheduler.schedulers.asyncio import AsyncIOScheduler
from apscheduler.triggers.cron import CronTrigger
from fastapi import FastAPI, HTTPException
from fastapi.staticfiles import StaticFiles
from loguru import logger
from starlette.responses import FileResponse

from .config import config
from .log import LOGGING_CONFIG
from .subscribe import counter, update

app = FastAPI()

# provider download
app.mount(f"/{config.urlprefix}/provider", StaticFiles(directory="data/provider"))


# profile download
@app.get(f"/{config.urlprefix}/profile" + "/{path}")
async def profile(path: str):
    path = path.rsplit(".", 1)[0] + ".yaml"
    logger.info(f"A request to download profile {path} was received")
    if path[:-5] not in config.profiles.keys():
        raise HTTPException(404, f"Profile {path} not found")
    resp = FileResponse(
        path=f"data/profile/{path}",
    )
    counter_info = await counter(path[:-5])
    if counter_info:
        resp.headers["subscription-userinfo"] = counter_info
    for h in config.headers:
        resp.headers[h] = config.headers[h]
    return resp


# manual update trigger
@app.get(f"/{config.urlprefix}/update")
async def _():
    logger.info("Update is triggered manually")
    error = await update()
    return str(error) or "update complete"


@app.on_event("startup")
async def startup_event():
    error = await update()
    if error:
        raise error
    logger.info(
        f"Starting up scheduler from crontab {config.update_cron} at timezone {config.update_tz}"
    )
    scheduler = AsyncIOScheduler()
    scheduler.add_job(
        update, CronTrigger.from_crontab(config.update_cron, config.update_tz)
    )
    scheduler.start()
    logger.info(
        f"Application startup complete, listening requests from {config.domian}/{config.urlprefix}/"
    )


def main():
    uvicorn.run(app, host=config.host, port=config.port, log_config=LOGGING_CONFIG)



import base64
import json
from datetime import datetime
from typing import List, Optional, Union

from pytz import timezone

from ..model.clash import SS, Vmess
from ..utils import Download
from .base64 import ss, vmess


async def counter(url, tz: Optional[str] = None):
    info = json.loads(await Download.content(url))
    download_ = info["bw_counter_b"]
    total = info["monthly_bw_limit_b"]
    timenow = datetime.now()
    if timenow.month == 12:
        month = 1
        year = timenow.year + 1
    else:
        month = timenow.month + 1
        year = timenow.year
    expire = int(
        datetime(
            year=year,
            month=month,
            day=info["bw_reset_day_of_month"],
            tzinfo=timezone(tz) if tz else None,
        ).timestamp()
    )

    return f"upload=0; download={download_}; total={total}; expire={expire}"


async def get(url: str) -> List[Union[SS, Vmess]]:
    bsubs = await Download.content(url)
    subs = base64.decodebytes(bsubs).decode().split("\n")
    proxies = []
    for sub in subs:
        if sub.startswith("ss://"):
            proxies.append(ss(sub))
        elif sub.startswith("vmess://"):
            proxies.append(vmess(sub))
    return proxies



from typing import List, Union

from loguru import logger

from ..config import config
from ..model.clash import SS, SSR, ClashTemplate, Snell, Socks5, Trojan, Vmess
from ..utils import Download
from . import clash, jms


async def _subs(
    subs: List[str],
) -> List[Union[SS, SSR, Vmess, Socks5, Snell, Trojan]]:
    proxies = []
    for name in subs:
        sub = config.subscribes[name]
        if sub.type == "jms":
            proxies += await jms.get(sub.url)
        if sub.type == "ClashSub":
            proxies += await clash.get_sub(sub.url)
        if sub.type == "ClashFile":
            proxies += await clash.get_file(sub.file)
    return proxies


async def update():
    logger.info("Start update profiles")
    try:
        rulesets = {}
        for profile in config.profiles:
            proxies = await _subs(config.profiles[profile].subs)
            logger.debug(
                f"Generating profile {profile} from template {config.profiles[profile].template}"
            )
            template = ClashTemplate.load(config.profiles[profile].template)
            clash = template.render(proxies)
            if clash.rule_providers:
                for provider in clash.rule_providers:
                    rulesets[provider] = clash.rule_providers[provider].url
                    clash.rule_providers[provider].url = "/".join(
                        [
                            config.domian,
                            config.urlprefix,
                            "provider",
                            f"{provider}.yaml",
                        ]
                    )
            clash.save(profile)
        await Download.provider(rulesets)
        logger.success("Update complete")

    except Exception as e:
        logger.critical(e)
        return e


async def counter(profile: str):
    if len(config.profiles[profile].subs) != 1:
        logger.warning("Subscribe(s) is not 1, counter function disabled")
        return ""
    sub = config.subscribes[config.profiles[profile].subs[0]]
    if sub.type == "jms" and sub.counter:
        return await jms.counter(sub.counter, sub.subtz)
    elif sub.type == "ClashSub":
        return await clash.counter(sub.url)
    return ""
